package com.develop.auth.ui

import android.text.InputFilter
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.EditText
import androidx.fragment.app.activityViewModels
import com.develop.auth.R
import com.develop.auth.databinding.FragmentMemberRegisterBinding
import com.develop.food.base.ui.ViewBindingFragment
import com.develop.food.base.utils.isEmail
import java.util.regex.Matcher
import java.util.regex.Pattern

class MemberRegisterFragment: ViewBindingFragment<FragmentMemberRegisterBinding>() {

    private val viewModel by activityViewModels<MemberViewModel>()

    override fun createViewBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentMemberRegisterBinding {
        return FragmentMemberRegisterBinding.inflate(inflater, container, false)
    }

    override fun onPostCreateView() {
        binding.tvRegister.setOnClickListener {
            if (!checkInputs()) {
                return@setOnClickListener
            }
            executeRegister(
                binding.etEmailId.text.toString(),
                binding.etPassword.text.toString()
            )
        }
        binding.tvLogin.setOnClickListener {
            parentFragmentManager
                .beginTransaction()
                .replace(R.id.fl_container, MemberLoginFragment())
                .commitAllowingStateLoss()
        }
        setEditTextInhibitInputSpeChat(binding.etPassword)
        setEditTextInhibitInputSpeChat(binding.etPasswordAgain)
    }


    private fun setEditTextInhibitInputSpeChat(editText: EditText) {
        val filter =
            InputFilter { source, start, end, dest, dstart, dend ->
                val speChat = "[-©®™℅¡¿¥£°^¢€%&-+()~`|•√π÷×¶∆`~!@#$%^=|':;',\\[\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？\"'_\\\\]"
                val pattern: Pattern = Pattern.compile(speChat)
                val matcher: Matcher = pattern.matcher(source.toString())
                if (matcher.find()) "" else null
            }
        editText.filters = arrayOf(filter)
    }

    private fun checkInputs(): Boolean {
        val inputEmailId = binding.etEmailId.text.toString()
        val inputPsd = binding.etPassword.text.toString()
        val inputPsdConfirm = binding.etPasswordAgain.text.toString()
        if (inputEmailId.isEmpty() || inputPsd.isEmpty()) {
            showToast(getString(R.string.please_fill_in))
            return false
        }

        if (!inputEmailId.isEmail()){
            showToast("Please input email")
            return false
        }

        if (!TextUtils.equals(inputPsd, inputPsdConfirm)) {
            showToast(getString(R.string.password_is_not_the_same))
            return false
        }
        if (inputPsd.length < 8) {
            showToast(getString(R.string.password_should_more_than))
            return false
        }
        if (!hasLetterAndDigit(inputPsd)) {
            showToast(getString(R.string.password_should_include))
            return false
        }
        return true
    }

    private fun executeRegister(emailId: String, password: String) {
        showDialog()
        viewModel.register(
            emailId, password
        ).observe(this) {
            dismissDialog()
            if (!it) {
                return@observe
            }
            parentFragmentManager
                .beginTransaction()
                .replace(R.id.fl_container, MemberProfileFragment(), "MemberProfileFragment")
                .commitAllowingStateLoss()
        }
    }

    private fun hasLetterAndDigit(str: String?): Boolean {
        if (str.isNullOrEmpty()) {
            return false
        }
        var hasLetter = false
        var hasDigit = false
        val array = str.toCharArray()
        for (char in array) {
            if (Character.isDigit(char)) {
                hasDigit = true
            } else if (Character.isLetter(char)) {
                hasLetter = true
            }
        }
        return hasDigit && hasLetter
    }
}