package com.develop.step.model

sealed class CookStatus(var isRunning: Boolean) {

    class Weight(
        val weightNum: Int
    ): CookStatus(false)

    class Temp(
        var settingTemp: Int,
        var currentTemp: Int = 0,
        isRunning: Boolean = false
    ): CookStatus(isRunning)

    class Time(
        var settingHours: Int,
        var settingMinutes: Int,
        var settingSeconds: Int,
        var currentHours: Int = 0,
        var currentMinutes: Int = 0,
        var currentSeconds: Int = 0,
        isRunning: Boolean = false
    ): CookStatus(isRunning) {

        fun settingTotalSeconds(): Short {
            val seconds = settingHours * 60 * 60 + settingMinutes * 60 + settingSeconds
            return seconds.toShort()
        }
    }

    class Speed(
        var settingSpeed: Int,
        var currentSpeed: Int,
        isRunning: Boolean = false
    ): CookStatus(isRunning)

    class Direction(
        var settingValue: Int,
        var currentValue: Int,
        isRunning: Boolean = false
    ): CookStatus(isRunning)
}